// $ANTLR 2.7.1: "expandedproblem.g" -> "ProblemParser.java"$


    package de.uka.ilkd.key.parser;

    import java.io.*;
    import java.util.Vector;
    import java.util.HashMap;
    import java.util.HashSet;
    import java.util.Iterator;

    import antlr.*;

    import de.uka.ilkd.key.logic.*;
    import de.uka.ilkd.key.logic.sort.*;
    import de.uka.ilkd.key.logic.op.*;
    import de.uka.ilkd.key.rule.metaconstruct.*;
    import de.uka.ilkd.key.java.JavaInfo;
    import de.uka.ilkd.key.java.JavaReader;
    import de.uka.ilkd.key.java.SchemaJavaReader;
    import de.uka.ilkd.key.java.visitor.ProgramVariableCollector;
    import de.uka.ilkd.key.proof.*;
    import de.uka.ilkd.key.rule.*;
    import de.uka.ilkd.key.java.visitor.*;



import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;
import antlr.collections.AST;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

public class ProblemParser extends antlr.LLkParser
       implements ProblemParserTokenTypes
 {


    static final Sort[] AN_ARRAY_OF_SORTS = new Sort[0];
    static final Term[] AN_ARRAY_OF_TERMS = new Term[0];

    protected JavaInfo javaInfo;
	
    protected Namespace var_ns  = new Namespace();
    protected Namespace func_ns = new Namespace();
    protected Namespace sort_ns = new Namespace();
    protected Namespace heur_ns = new Namespace();

    // these variables are set if a file is read in step by
    // step. This used when reading in LDTs because of cyclic
    // dependencies.
    protected boolean skip_functions;
    protected boolean skip_predicates;
    protected boolean skip_sorts;
    protected boolean skip_schemavariables;
    protected boolean skip_heuristics;
    protected UseInsteadMap useInstead=new UseInsteadMap();	

    protected int lineOffset=0;
    protected int colOffset=0;
	

    protected SetOfTaclet taclets = SetAsListOfTaclet.EMPTY_SET;

    private static ListOfRulevariable NO_RULE_VARIABLES =
	SLListOfRulevariable.EMPTY_LIST;
    private Term a=null;

    protected TermFactory tf = TermFactory.DEFAULT;
    boolean schematicJava;	
    boolean allowSchemaVars;

    public boolean errors = false;


    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo) {
	this(lexer);
	this.javaInfo = javaInfo;
    }

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo, UseInsteadMap uim) {
	this(lexer, javaInfo);
	this.useInstead=uim;
    }

    public ProblemParser(TokenStream lexer, 
			 String filename,
			 TermFactory tf,
			 JavaInfo javaInfo) {
	this(lexer, javaInfo);
	setFilename(filename);
	this.tf = tf;	
    }
	

    public ProblemParser(TokenStream lexer, 
			 String filename,
			 TermFactory tf,
			 JavaInfo javaInfo,
			 Namespace var_ns,
			 Namespace func_ns,
			 Namespace sort_ns,
			 Namespace heur_ns) {
	this(lexer, javaInfo);
	setFilename(filename);
	this.tf = tf;

	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
	this.var_ns  = var_ns;
	this.heur_ns = heur_ns;	
    }

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,
			 Namespace var_ns,
			 Namespace func_ns,
			 Namespace sort_ns,
			 Namespace heur_ns) {
	this(lexer, javaInfo);

	this.tf = tf;

	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
	this.var_ns  = var_ns;
	this.heur_ns = heur_ns;	
    }


    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,
			 NamespaceSet ns) {

		this(lexer, javaInfo, ns.variables(), ns.functions(), 
			ns.sorts(), ns.heuristics());
	}

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,
			 NamespaceSet ns,
			 UseInsteadMap uim) {
		this (lexer, javaInfo, ns);
		this.useInstead=uim;
	}


    protected void setSkips(boolean svars, boolean funcs, 
			    boolean preds, boolean sorts,
			    boolean heurs) {

	skip_schemavariables = svars;
	skip_functions       = funcs;
	skip_predicates      = preds;
	skip_sorts           = sorts;
	skip_heuristics      = heurs;

    }

    protected void skipFuncs() {
	skip_functions = true;
    }

    protected void skipPreds() {
	skip_predicates = true;
    }

    protected void skipVars() {
	skip_schemavariables = true;
    }

    protected void skipSorts() {
	skip_sorts = true;
    }

    protected void skipHeur() {
	skip_heuristics = true;
    }
	
    /** returns the java information object */
    public JavaInfo getJavaInfo() {
	return javaInfo;
    }


    /** parses a problem but without reading the declarations of
     * sorts, functions and predicates. These have to be given
     * explicitly.
     * @param func_ns the Namespace with the function and predicate
     * symbols to be used
     * @param sort_ns the Namespace with the sorts to be used
     * @param heur_ns the Namespace with the heuristics to be used
     * the heuristics of the current problem file will be added	
     */ 
    public Term parseProblemGivenNamespaces
	(Namespace func_ns, 
	Namespace sort_ns,
	Namespace heur_ns) 
	throws antlr.RecognitionException, antlr.TokenStreamException
	
	{
	// restore known state
	setSkips(false, false, false, false, false);

	// set namespaces
	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
	this.heur_ns = heur_ns;
	// skip decls
	skipSorts();
	skipFuncs();
	skipPreds();	
	
	return problem();
    }

    public void addSort(Sort s) {
	sort_ns.add(s);
    }

    public void addSorts(Namespace ns) {
	IteratorOfNamed it=ns.elements().iterator();
	while (it.hasNext()) {
	    addSort((Sort)it.next());
	}
    }

    public void addFunction(Function f) {
	func_ns.add(f);
    }


    public void addFunctions(Namespace ns) {
	IteratorOfNamed it=ns.elements().iterator();
	while (it.hasNext()) {
	    addFunction((Function)it.next());
	}
    }

    public Namespace getSorts() {
	return sort_ns;
    }

    public Namespace getVariables() {
	return var_ns;
    }

    public Namespace getFunctions() {
	return func_ns;
    }

    public Namespace getHeuristics() {
	return heur_ns;
    }

    public NamespaceSet namespaces() {
	NamespaceSet ns = new NamespaceSet();
	ns.setVariables(getVariables());
	ns.setFunctions(getFunctions());
	ns.setSorts(getSorts());
	ns.setHeuristics(getHeuristics());
	return ns;
    }

    public SetOfTaclet getTaclets() {
	return taclets;
    }

    protected int getLine() {
	int line = -1;
	try {
	    line = LT(0).getLine()+lineOffset;
	} 
	catch (Exception e) {
	    System.err.println("No further token in stream");
	}
	return line;
    }   

    protected int getColumn() {
	int 	col = -1;
	try {
	    col = LT(0).getColumn()+colOffset;
	} 
	catch (Exception e) {
	    System.err.println("No further token in stream");
	}
	return col;
    }       

    protected TermSymbol progVar(Token id,Sort s) {
	TermSymbol v = (TermSymbol)
	    var_ns.lookup(new Name(id.getText()));
	if (v == null) {
	    v = new ProgramVariable(new ProgramElementName(id.getText()), s);
	    var_ns=var_ns.extended(v);
	}
	return v;
    }

    protected static HashSet progVars(JavaBlock jb) {
	if (jb==JavaBlock.EMPTY_JAVABLOCK) {
		return new HashSet();
	}	
       ProgramVariableCollector pvc
	  = new ProgramVariableCollector(jb.program());
	pvc.start();
	return pvc.result();
    }

    protected void bindProgVars(HashSet progVars) {
	var_ns=new Namespace(var_ns);
	Iterator it=progVars.iterator();
	while (it.hasNext()) {
	    var_ns.add((ProgramVariable)it.next());
	}
    }

    protected void unbindProgVars() {
	var_ns = var_ns.parent();
    }


    protected LogicVariable bindVar(Token id,Sort s) {
	LogicVariable v=new LogicVariable(new Name(id.getText()), s);
	var_ns=var_ns.extended(v);
	return v;
    }
	
    protected void unbindVars() {
	var_ns = var_ns.parent();
    }




    protected Term termForParsedVariable(ParsableVariable v) 
	throws antlr.SemanticException 
	{
	    if ( v instanceof LogicVariable ) {
		return tf.createVariableTerm((LogicVariable)v);
	    } else if ( allowSchemaVars
			&& v instanceof SchemaVariable ) {
		return tf.createVariableTerm((SchemaVariable)v);
	    } else if ( v instanceof ProgramVariable ) {
		return tf.createVariableTerm((ProgramVariable)v);
	    }
	    if ( allowSchemaVars ) {
		throw new antlr.SemanticException(
						  ""+ v +" is not a program or logic or schema variable");
	    } else {
		throw new antlr.SemanticException(
						  ""+ v +" is not a logic variable "
						  +"or program variable");
	    }
	}



    private TacletBuilder createTacletBuilderFor(Object find) 
	throws InvalidFindException {
	if ( find == null ) {
	    return new NoFindTacletBuilder();
	} else if ( find instanceof Term ) {
	    return new RewriteTacletBuilder().setFind((Term)find);
	} else if ( find instanceof Sequent ) {
	    Sequent findSeq = (Sequent) find;
	    if ( findSeq == Sequent.EMPTY_SEQUENT ) {
		return new NoFindTacletBuilder();
	    } else if (   findSeq.antecedent().size() == 1
			  && findSeq.succedent().size() == 0 ) {
		Term findFma = findSeq.antecedent().get(0).formula();
		return new AntecTacletBuilder().setFind(findFma);
	    } else if (   findSeq.antecedent().size() == 0
			  && findSeq.succedent().size() == 1 ) {
		Term findFma = findSeq.succedent().get(0).formula();
		return new SuccTacletBuilder().setFind(findFma);
	    } else {
		throw new InvalidFindException
		    ("Unknown find-sequent (perhaps null?):"+findSeq,
		     getFilename(), getLine(), getColumn());	
	    }
	} else {
	    throw new RuntimeException("createTacletBuilderFor called with"
				       + find.getClass().getName() + "object");
	}
    }	    



    private void addGoalTemplate(TacletBuilder b,
                                 String id,
				 Object rwObj,
				 Sequent addSeq,
				 ListOfTaclet addRList,
				 SetOfSchemaVariable pvs) 
	throws SemanticException
	{
	    TacletGoalTemplate gt = null;
	    if ( rwObj == null ) {
		// there is no replacewith, so we take
		gt = new TacletGoalTemplate(addSeq,
					  addRList,
					  NO_RULE_VARIABLES,
					  pvs);
	    } else {
		if ( b instanceof NoFindTacletBuilder ) {
		    // there is a replacewith without a find.
		    throw
			new UnfittingReplacewithException
			("Replacewith without find", getFilename(),
			 getLine(), getColumn());
		} else if ( b instanceof SuccTacletBuilder
			    || b instanceof AntecTacletBuilder ) {
		    if ( rwObj instanceof Sequent ) {
			gt = new AntecSuccTacletGoalTemplate(addSeq,
							   addRList,
							   NO_RULE_VARIABLES,
							   (Sequent)rwObj, pvs); 
		    } else {
			throw new UnfittingReplacewithException
			    ("Replacewith in a Antec-or SuccTaclet has "+
			     "to contain a sequent (not a term)", 
			     getFilename(), getLine(), getColumn());
		    }
		} else if ( b instanceof RewriteTacletBuilder ) {
		    if ( rwObj instanceof Term ) {
			gt = new RewriteTacletGoalTemplate(addSeq,
							 addRList,
							 NO_RULE_VARIABLES,
							 (Term)rwObj, pvs);  
		    } else {
			throw new UnfittingReplacewithException
			    ("Replacewith in a RewriteTaclet has "+
			     "to contain a term (not a sequent)", 
			     getFilename(), getLine(), getColumn());
		    }
		}
	    }
            gt.setName(id);
	    b.addTacletGoalTemplate(gt);
	}
							
	
    public static void main(String[] args) {
	ProblemParser p = 
           new ProblemParser(new KeYLexer(
              new BufferedReader(new InputStreamReader(System.in))));
	try {
	    Term fma = p.problem();
	    System.out.println("Parsed problem:\n"+fma);
	    System.out.println("Parsed Taclets:\n"+p.getTaclets());
	} catch (TokenStreamException e) {
	    System.out.println("Lexing error");
	    e.printStackTrace();
	} catch (RecognitionException e) {
	    System.out.println("Parsing error");
	    e.printStackTrace();
	}
    }


    protected JavaBlock getJavaBlock(Token t) throws antlr.SemanticException {
	String s=t.getText();
	de.uka.ilkd.key.java.JavaReader jr;
	try {
		if (schematicJava) {
		    jr = new de.uka.ilkd.key.java.SchemaRecoder2KeY
			(getJavaInfo().rec2key(), getJavaInfo().getTypeConverter());
		    ((SchemaJavaReader)jr).setSVNamespace(var_ns);
		    return jr.readBlockWithEmptyContext(s);
		} else {
		    jr = new de.uka.ilkd.key.java.Recoder2KeY(javaInfo.rec2key(),
						getJavaInfo().getTypeConverter());
		    return jr.readBlockWithProgramVariables(var_ns, s);	
		}
	} catch (de.uka.ilkd.key.java.ConvertException e) {
		if (s.length()>10) {
			s=s.substring(0,10)+"...";
		}
		if (e.parseException()!=null
		    &&	e.parseException().currentToken != null
		    &&	e.parseException().currentToken.next != null) {		  
		  lineOffset=e.parseException().currentToken.next.beginLine;
		  colOffset=e.parseException().currentToken.next.beginColumn;
		  e.parseException().currentToken.next.beginLine=getLine()-1;
		  e.parseException().currentToken.next.beginColumn=getColumn();
		  throw new JavaParserException(e.parseException().getMessage(), t, 
			getFilename(), -1, -1);  // row/columns already in text
		}		
		throw new JavaParserException(e.getMessage(), t, getFilename());
	}		
    }

protected ProblemParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public ProblemParser(TokenBuffer tokenBuf) {
  this(tokenBuf,2);
}

protected ProblemParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public ProblemParser(TokenStream lexer) {
  this(lexer,2);
}

public ProblemParser(ParserSharedInputState state) {
  super(state,2);
  tokenNames = _tokenNames;
}

	public final void top() throws RecognitionException, TokenStreamException {
		
		
		if ( inputState.guessing==0 ) {
		}
		a=problem();
	}
	
	public final Term  problem() throws RecognitionException, TokenStreamException {
		 Term a = null ;
		
		
		Taclet s = null;
		String st = null;
		
		
		{
		st=java();
		}
		decls();
		{
		switch ( LA(1)) {
		case RULES:
		{
			match(RULES);
			match(LBRACE);
			if ( inputState.guessing==0 ) {
				allowSchemaVars = true; 
				schematicJava=true;
				
			}
			{
			_loop6:
			do {
				if ((LA(1)==IDENT)) {
					s=taclet();
					match(SEMI);
					if ( inputState.guessing==0 ) {
						
						try {
						taclets=taclets.addUnique(s);
						} catch(de.uka.ilkd.key.collection.NotUniqueException e) {
						throw new RuntimeException(e.toString());
						}
						
					}
				}
				else {
					break _loop6;
				}
				
			} while (true);
			}
			match(RBRACE);
			break;
		}
		case EOF:
		case PROBLEM:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			allowSchemaVars = false; 
			schematicJava=false;
			
		}
		{
		switch ( LA(1)) {
		case PROBLEM:
		{
			match(PROBLEM);
			match(LBRACE);
			a=formula();
			match(RBRACE);
			break;
		}
		case EOF:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final String  java() throws RecognitionException, TokenStreamException {
		String s = null;
		
		Token  l = null;
		StringBuffer b=new StringBuffer();
		
		{
		switch ( LA(1)) {
		case JAVA:
		{
			{
			match(JAVA);
			match(LBRACE);
			{
			int _cnt12=0;
			_loop12:
			do {
				switch ( LA(1)) {
				case IDENT:
				{
					l = LT(1);
					match(IDENT);
					if ( inputState.guessing==0 ) {
						b.append(l.getText());
					}
					break;
				}
				case SLASH:
				{
					match(SLASH);
					if ( inputState.guessing==0 ) {
						b.append("/");
					}
					break;
				}
				default:
				{
					if ( _cnt12>=1 ) { break _loop12; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				}
				_cnt12++;
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				s=b.toString();
			}
			match(RBRACE);
			}
			break;
		}
		case EOF:
		case HEURISTICS:
		case SORTS:
		case SCHEMA:
		case PREDICATES:
		case FUNCTIONS:
		case RULES:
		case PROBLEM:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return s;
	}
	
	public final void decls() throws RecognitionException, TokenStreamException {
		
		
		{
		_loop131:
		do {
			switch ( LA(1)) {
			case SORTS:
			{
				sort_decls();
				break;
			}
			case SCHEMA:
			{
				schema_var_decls();
				break;
			}
			case PREDICATES:
			{
				pred_decls();
				break;
			}
			case FUNCTIONS:
			{
				func_decls();
				break;
			}
			case HEURISTICS:
			{
				heur_decls();
				break;
			}
			default:
			{
				break _loop131;
			}
			}
		} while (true);
		}
	}
	
	public final Taclet  taclet() throws RecognitionException, TokenStreamException {
		Taclet r;
		
		Token  name = null;
		
			Sequent ifSeq = Sequent.EMPTY_SEQUENT;
			Object  find = null;
			r = null;
			TacletBuilder b = null;
		
		
		name = LT(1);
		match(IDENT);
		match(LBRACE);
		{
		switch ( LA(1)) {
		case IF:
		{
			match(IF);
			match(LPAREN);
			ifSeq=seq();
			match(RPAREN);
			break;
		}
		case FIND:
		case VARCOND:
		case CLOSE:
		case REPLACEWITH:
		case ADD:
		case ADDRULES:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		switch ( LA(1)) {
		case FIND:
		{
			match(FIND);
			match(LPAREN);
			find=termorseq();
			match(RPAREN);
			break;
		}
		case VARCOND:
		case CLOSE:
		case REPLACEWITH:
		case ADD:
		case ADDRULES:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			
						b = createTacletBuilderFor(find);
						b.setName(new Name(name.getText()));
						b.setIfseq(ifSeq);
					
		}
		{
		switch ( LA(1)) {
		case VARCOND:
		{
			match(VARCOND);
			match(LPAREN);
			varexplist(b);
			match(RPAREN);
			break;
		}
		case CLOSE:
		case REPLACEWITH:
		case ADD:
		case ADDRULES:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		goalspecs(b);
		modifiers(b);
		match(RBRACE);
		if ( inputState.guessing==0 ) {
			r = b.getTaclet();
		}
		return r;
	}
	
	public final Term  formula() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		a=term();
		if ( inputState.guessing==0 ) {
			
						if ( a.sort() != Sort.FORMULA ) {
							throw new antlr.SemanticException(
								"Just Parsed a Term where a Formula was expected");
						}
					
		}
		return a;
	}
	
	public final void proof(
		ProblemLoader prl
	) throws RecognitionException, TokenStreamException {
		
		
		{
		switch ( LA(1)) {
		case PROOF:
		{
			match(PROOF);
			match(LBRACE);
			pseudosexpr(prl);
			match(RBRACE);
			break;
		}
		case EOF:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
	}
	
	public final void pseudosexpr(
		ProblemLoader prl
	) throws RecognitionException, TokenStreamException {
		
		Token  str = null;
		char eid='0';
		
		if ((LA(1)==LPAREN) && (LA(2)==RPAREN)) {
			match(LPAREN);
			match(RPAREN);
		}
		else if ((LA(1)==LPAREN) && (_tokenSet_0.member(LA(2)))) {
			match(LPAREN);
			eid=expreid();
			str = LT(1);
			match(STRING_LITERAL);
			if ( inputState.guessing==0 ) {
				prl.beginExpr(eid,str.getText());
			}
			{
			_loop17:
			do {
				if ((LA(1)==LPAREN)) {
					pseudosexpr(prl);
				}
				else {
					break _loop17;
				}
				
			} while (true);
			}
			if ( inputState.guessing==0 ) {
				prl.endExpr(eid);
			}
			match(RPAREN);
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
	}
	
	public final char  expreid() throws RecognitionException, TokenStreamException {
		 char eid = '0' ;
		
		
		switch ( LA(1)) {
		case FORMULA:
		{
			match(FORMULA);
			if ( inputState.guessing==0 ) {
				eid='f';
			}
			break;
		}
		case TERM:
		{
			match(TERM);
			if ( inputState.guessing==0 ) {
				eid='t';
			}
			break;
		}
		case RULE:
		{
			match(RULE);
			if ( inputState.guessing==0 ) {
				eid= 'r';
			}
			break;
		}
		case BRANCH:
		{
			match(BRANCH);
			if ( inputState.guessing==0 ) {
				eid='b';
			}
			break;
		}
		case IDENT:
		{
			match(IDENT);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return eid;
	}
	
	public final Sequent  seq() throws RecognitionException, TokenStreamException {
		Sequent s;
		
		Semisequent ant,suc; s = null;
		
		ant=semisequent();
		match(SEQARROW);
		suc=semisequent();
		if ( inputState.guessing==0 ) {
			s = Sequent.createSequent(ant, suc);
		}
		return s;
	}
	
	public final Object  termorseq() throws RecognitionException, TokenStreamException {
		Object o;
		
		
			Term head = null;
			Sequent s = null;
			Semisequent ss = null;
			o = null; 
		
		
		switch ( LA(1)) {
		case FORALL:
		case EXISTS:
		case EPSILON:
		case TRUE:
		case FALSE:
		case NOT:
		case IN_TYPE:
		case META_IN_LONG:
		case META_IN_INT:
		case META_IN_SHORT:
		case META_IN_BYTE:
		case META_ADD:
		case META_SUB:
		case META_MUL:
		case META_DIV:
		case META_MOD:
		case META_LESS:
		case META_GREATER:
		case META_LEQ:
		case META_GEQ:
		case META_EQ:
		case META_INT_UNARY_MINUS:
		case META_INT_ADD:
		case META_INT_SUB:
		case META_INT_MUL:
		case META_INT_DIV:
		case META_INT_MOD:
		case META_INT_AND:
		case META_INT_OR:
		case META_INT_XOR:
		case META_INT_SHIFTRIGHT:
		case META_INT_SHIFTLEFT:
		case META_INT_UNSIGNEDSHIFTRIGHT:
		case META_LONG_UNARY_MINUS:
		case META_LONG_ADD:
		case META_LONG_SUB:
		case META_LONG_MUL:
		case META_LONG_DIV:
		case META_LONG_MOD:
		case META_LONG_AND:
		case META_LONG_OR:
		case META_LONG_XOR:
		case META_LONG_SHIFTRIGHT:
		case META_LONG_SHIFTLEFT:
		case META_LONG_UNSIGNEDSHIFTRIGHT:
		case META_MODULO_BYTE:
		case META_MODULO_SHORT:
		case META_MODULO_INTEGER:
		case META_MODULO_LONG:
		case LPAREN:
		case LBRACE:
		case DIAMOND:
		case BOX:
		case IDENT:
		{
			head=term();
			{
			switch ( LA(1)) {
			case COMMA:
			{
				match(COMMA);
				s=seq();
				break;
			}
			case SEQARROW:
			{
				match(SEQARROW);
				ss=semisequent();
				break;
			}
			case RPAREN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				
							if ( s == null ) {
								if ( ss == null ) {
									// Just a term
									o = head;
								} else {
									// A sequent with only head in the antecedent.
									Semisequent ant = Semisequent.EMPTY_SEMISEQUENT;
									ant = ant.insertFirst(
										new ConstrainedFormula(head, Constraint.BOTTOM));
									o = Sequent.createSequent(ant,ss);
								}
							} else {
								// A sequent.  Prepend head to the antecedent.
								Semisequent newAnt = s.antecedent();
								newAnt = newAnt .insertFirst(
									new ConstrainedFormula(head, Constraint.BOTTOM));
								o = Sequent.createSequent(newAnt,s.succedent());
							}
						
			}
			break;
		}
		case SEQARROW:
		{
			match(SEQARROW);
			ss=semisequent();
			if ( inputState.guessing==0 ) {
				
							o = Sequent.createSequent(Semisequent.EMPTY_SEMISEQUENT,ss);
						
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return o;
	}
	
	public final void varexplist(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		
		varexp(b);
		{
		_loop33:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				varexp(b);
			}
			else {
				break _loop33;
			}
			
		} while (true);
		}
	}
	
	public final void goalspecs(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		
		switch ( LA(1)) {
		case CLOSE:
		{
			match(CLOSE);
			match(GOAL);
			break;
		}
		case REPLACEWITH:
		case ADD:
		case ADDRULES:
		{
			goalspec(b);
			{
			_loop41:
			do {
				if ((LA(1)==SEMI)) {
					match(SEMI);
					goalspec(b);
				}
				else {
					break _loop41;
				}
				
			} while (true);
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
	}
	
	public final void modifiers(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		Token  dname = null;
		Token  htext = null;
		
		{
		_loop25:
		do {
			switch ( LA(1)) {
			case HEURISTICS:
			{
				heuristics(b);
				break;
			}
			case NONINTERACTIVE:
			{
				match(NONINTERACTIVE);
				if ( inputState.guessing==0 ) {
					b.setNoninteractive(true);
				}
				break;
			}
			case RECURSIVE:
			{
				match(RECURSIVE);
				if ( inputState.guessing==0 ) {
					b.setRecursive(true);
				}
				break;
			}
			case DISPLAYNAME:
			{
				match(DISPLAYNAME);
				dname = LT(1);
				match(STRING_LITERAL);
				if ( inputState.guessing==0 ) {
					b.setDisplayName(dname.getText());
				}
				break;
			}
			case HELPTEXT:
			{
				match(HELPTEXT);
				htext = LT(1);
				match(STRING_LITERAL);
				if ( inputState.guessing==0 ) {
					b.setHelpText(htext.getText());
				}
				break;
			}
			default:
			{
				break _loop25;
			}
			}
		} while (true);
		}
	}
	
	public final void heuristics(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		
		match(HEURISTICS);
		match(LPAREN);
		heuristic(b);
		{
		_loop61:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				heuristic(b);
			}
			else {
				break _loop61;
			}
			
		} while (true);
		}
		match(RPAREN);
	}
	
	public final Semisequent  semisequent() throws RecognitionException, TokenStreamException {
		Semisequent ss;
		
		
			Term head = null;
			ss = Semisequent.EMPTY_SEMISEQUENT; 
		
		
		switch ( LA(1)) {
		case RPAREN:
		case SEQARROW:
		{
			break;
		}
		case FORALL:
		case EXISTS:
		case EPSILON:
		case TRUE:
		case FALSE:
		case NOT:
		case IN_TYPE:
		case META_IN_LONG:
		case META_IN_INT:
		case META_IN_SHORT:
		case META_IN_BYTE:
		case META_ADD:
		case META_SUB:
		case META_MUL:
		case META_DIV:
		case META_MOD:
		case META_LESS:
		case META_GREATER:
		case META_LEQ:
		case META_GEQ:
		case META_EQ:
		case META_INT_UNARY_MINUS:
		case META_INT_ADD:
		case META_INT_SUB:
		case META_INT_MUL:
		case META_INT_DIV:
		case META_INT_MOD:
		case META_INT_AND:
		case META_INT_OR:
		case META_INT_XOR:
		case META_INT_SHIFTRIGHT:
		case META_INT_SHIFTLEFT:
		case META_INT_UNSIGNEDSHIFTRIGHT:
		case META_LONG_UNARY_MINUS:
		case META_LONG_ADD:
		case META_LONG_SUB:
		case META_LONG_MUL:
		case META_LONG_DIV:
		case META_LONG_MOD:
		case META_LONG_AND:
		case META_LONG_OR:
		case META_LONG_XOR:
		case META_LONG_SHIFTRIGHT:
		case META_LONG_SHIFTLEFT:
		case META_LONG_UNSIGNEDSHIFTRIGHT:
		case META_MODULO_BYTE:
		case META_MODULO_SHORT:
		case META_MODULO_INTEGER:
		case META_MODULO_LONG:
		case LPAREN:
		case LBRACE:
		case DIAMOND:
		case BOX:
		case IDENT:
		{
			head=term();
			{
			switch ( LA(1)) {
			case COMMA:
			{
				match(COMMA);
				ss=semisequent();
				break;
			}
			case RPAREN:
			case SEQARROW:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if ( inputState.guessing==0 ) {
				ss = ss.insertFirst(new ConstrainedFormula(head, Constraint.BOTTOM));
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return ss;
	}
	
	public final Term  term() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		a=term20();
		return a;
	}
	
	public final void varexp(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		
			ParsableVariable x;
			ParsableVariable y;
			Sort s = null;
		
		
		{
		if ((LA(1)==TYPEOF)) {
			{
			match(TYPEOF);
			match(LPAREN);
			y=var_id();
			match(RPAREN);
			x=var_id();
			match(NEW);
			}
			if ( inputState.guessing==0 ) {
				b.addVarsNew((SchemaVariable) x, (SchemaVariable) y);
			}
		}
		else if ((LA(1)==IDENT) && (LA(2)==IDENT)) {
			{
			s=sort_id_check(true);
			x=var_id();
			match(NEW);
			}
			if ( inputState.guessing==0 ) {
				b.addVarsNew((SchemaVariable) x, s);
			}
		}
		else if ((LA(1)==IDENT) && (LA(2)==NOT)) {
			{
			x=var_id();
			match(NOT);
			match(FREE);
			match(IN);
			y=var_id();
			}
			if ( inputState.guessing==0 ) {
				b.addVarsNotFreeIn((SchemaVariable)x,(SchemaVariable)y);
			}
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
	}
	
	public final ParsableVariable  var_id() throws RecognitionException, TokenStreamException {
		ParsableVariable v = null;
		
		Token  id = null;
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
						v = (ParsableVariable) var_ns.lookup(new Name(id.getText()));
						if (v == null) {
							throw new NotDeclException("variable", id, 
										   getFilename());
						}
					
		}
		return v;
	}
	
	public final Sort  sort_id_check(
		boolean checkSort
	) throws RecognitionException, TokenStreamException {
		Sort s = null;
		
		Token  id = null;
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
				String idtext=(useInstead.get(id.getText()));
						s= (Sort) sort_ns.lookup(new Name(idtext));
						if (checkSort && s == null) {
						    throw 
							new NotDeclException("sort", id, 
									     getFilename());
						}
					
		}
		return s;
	}
	
	public final void goalspec(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
			Object rwObj = null;
			Sequent addSeq = Sequent.EMPTY_SEQUENT;
			ListOfTaclet addRList = SLListOfTaclet.EMPTY_LIST;
			SetOfSchemaVariable addpv = SetAsListOfSchemaVariable.EMPTY_SET;
		String name = null;
		
		
		{
		switch ( LA(1)) {
		case REPLACEWITH:
		{
			{
			match(REPLACEWITH);
			{
			switch ( LA(1)) {
			case STRING_LITERAL:
			{
				id = LT(1);
				match(STRING_LITERAL);
				break;
			}
			case LPAREN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(LPAREN);
			rwObj=termorseq();
			match(RPAREN);
			{
			switch ( LA(1)) {
			case ADD:
			{
				addSeq=add();
				break;
			}
			case NONINTERACTIVE:
			case RECURSIVE:
			case DISPLAYNAME:
			case HELPTEXT:
			case ADDRULES:
			case ADDPROGVARS:
			case HEURISTICS:
			case SEMI:
			case RBRACE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case ADDRULES:
			{
				addRList=addrules();
				break;
			}
			case NONINTERACTIVE:
			case RECURSIVE:
			case DISPLAYNAME:
			case HELPTEXT:
			case ADDPROGVARS:
			case HEURISTICS:
			case SEMI:
			case RBRACE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case ADDPROGVARS:
			{
				addpv=addprogvar();
				break;
			}
			case NONINTERACTIVE:
			case RECURSIVE:
			case DISPLAYNAME:
			case HELPTEXT:
			case HEURISTICS:
			case SEMI:
			case RBRACE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			}
			break;
		}
		case ADD:
		{
			{
			addSeq=add();
			{
			switch ( LA(1)) {
			case ADDRULES:
			{
				addRList=addrules();
				break;
			}
			case NONINTERACTIVE:
			case RECURSIVE:
			case DISPLAYNAME:
			case HELPTEXT:
			case HEURISTICS:
			case SEMI:
			case RBRACE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			}
			break;
		}
		case ADDRULES:
		{
			{
			addRList=addrules();
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			
				if (id != null) name = id.getText();
						addGoalTemplate(b,name,rwObj,addSeq,addRList,addpv);
					
		}
	}
	
	public final Sequent  add() throws RecognitionException, TokenStreamException {
		Sequent s;
		
		s = null;
		
		match(ADD);
		match(LPAREN);
		s=seq();
		match(RPAREN);
		return s;
	}
	
	public final ListOfTaclet  addrules() throws RecognitionException, TokenStreamException {
		ListOfTaclet lor;
		
		lor = null;
		
		match(ADDRULES);
		match(LPAREN);
		lor=tacletlist();
		match(RPAREN);
		return lor;
	}
	
	public final SetOfSchemaVariable  addprogvar() throws RecognitionException, TokenStreamException {
		SetOfSchemaVariable pvs;
		
		pvs = null;
		
		match(ADDPROGVARS);
		match(LPAREN);
		pvs=pvset();
		match(RPAREN);
		return pvs;
	}
	
	public final ListOfTaclet  tacletlist() throws RecognitionException, TokenStreamException {
		ListOfTaclet lor;
		
		
			Taclet head = null;
			lor = SLListOfTaclet.EMPTY_LIST; 
		
		
		head=taclet();
		{
		switch ( LA(1)) {
		case RPAREN:
		{
			break;
		}
		case COMMA:
		{
			match(COMMA);
			lor=tacletlist();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			lor = lor.prepend(head);
		}
		return lor;
	}
	
	public final SetOfSchemaVariable  pvset() throws RecognitionException, TokenStreamException {
		SetOfSchemaVariable pvs;
		
		
		ParsableVariable pv = null;
			pvs = SetAsListOfSchemaVariable.EMPTY_SET;
		
		
		pv=var_id();
		{
		switch ( LA(1)) {
		case RPAREN:
		{
			break;
		}
		case COMMA:
		{
			match(COMMA);
			pvs=pvset();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			pvs = pvs.add
										((SchemaVariable)pv);
		}
		return pvs;
	}
	
	public final void heuristic(
		TacletBuilder b
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
						Heuristic h = (Heuristic) heur_ns.lookup(new Name(id.getText()));
						if (h == null) {
							throw new NotDeclException("heuristic", id,
										   getFilename());
						}
						b.addHeuristic(h);
					
		}
	}
	
	public final Term  bindingterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		a=sv_or_local_bindingterm();
		return a;
	}
	
	public final Term  sv_or_local_bindingterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Token  v0 = null;
		Token  v1 = null;
		
			Operator op = null;
			QuantifiableVariable v = null;
			TermSymbol ts, pv = null;
			Term a1,a2;
			Term a0=null;
			Sort s = null;
			boolean bound = false;
		
		
		switch ( LA(1)) {
		case FORALL:
		case EXISTS:
		case EPSILON:
		{
			{
			{
			switch ( LA(1)) {
			case EPSILON:
			{
				match(EPSILON);
				if ( inputState.guessing==0 ) {
					op = Op.EPSILON;
				}
				break;
			}
			case FORALL:
			{
				match(FORALL);
				if ( inputState.guessing==0 ) {
					op = Op.ALL;
				}
				break;
			}
			case EXISTS:
			{
				match(EXISTS);
				if ( inputState.guessing==0 ) {
					op = Op.EX;
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			v0 = LT(1);
			match(IDENT);
			{
			switch ( LA(1)) {
			case COLON:
			{
				match(COLON);
				s=sort_id();
				if ( inputState.guessing==0 ) {
					
									v=bindVar(v0,s);
									bound = true;
								
				}
				break;
			}
			case DOT:
			{
				if ( inputState.guessing==0 ) {
					
									ts = (TermSymbol) var_ns.lookup(new Name(v0.getText()));
									if (   ts == null
									    || ts instanceof LogicVariable ) {
										throw new SemanticException(
											"Quantified variables need a sort.");
									}
									if ( ! (ts instanceof SchemaVariable && 
										((SchemaVariable)ts).isVariableSV())) {
										throw new SemanticException(
											""+ts+" is not allowed in a quantifier.");
									}
									v = (SchemaVariable) ts;
									bound = false;
								
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(DOT);
			a1=term60();
			if ( inputState.guessing==0 ) {
				
							if ( op == Op.EPSILON ) {
								a = tf.createEpsilonTerm(v,a1);
							} else {
								a = tf.createQuantifierTerm((Quantifier)op, v,a1);
							}
							if (bound) {unbindVars();}
						
			}
			}
			break;
		}
		case LBRACE:
		{
			{
			match(LBRACE);
			{
			boolean synPredMatched71 = false;
			if (((_tokenSet_1.member(LA(1))) && (LA(2)==COLON||LA(2)==DOT||LA(2)==LPAREN))) {
				int _m71 = mark();
				synPredMatched71 = true;
				inputState.guessing++;
				try {
					{
					match(IDENT);
					match(DOT);
					match(IDENT);
					match(ASSIGN);
					}
				}
				catch (RecognitionException pe) {
					synPredMatched71 = false;
				}
				rewind(_m71);
				inputState.guessing--;
			}
			if ( synPredMatched71 ) {
				{
				a0=attrterm();
				match(ASSIGN);
				a1=term60();
				match(RBRACE);
				a2=term60();
				if ( inputState.guessing==0 ) {
					
								a = tf.createUpdateTerm(a0,a1,a2);			
							
				}
				}
			}
			else if ((LA(1)==IDENT) && (_tokenSet_2.member(LA(2)))) {
				{
				v1 = LT(1);
				match(IDENT);
				{
				switch ( LA(1)) {
				case ASSIGN:
				{
					{
					match(ASSIGN);
					a1=term();
					match(RBRACE);
					if ( inputState.guessing==0 ) {
							pv = progVar(v1,a1.sort());
					}
					a2=term60();
					if ( inputState.guessing==0 ) {
						
										if (pv instanceof ProgramVariable) {
											a = tf.createUpdateTerm((ProgramVariable)pv,a1,a2);
										} else {
											a = tf.createUpdateTerm((SchemaVariable)pv,a1,a2);
										} 
									
					}
					}
					break;
				}
				case FORALL:
				case EXISTS:
				case EPSILON:
				case TRUE:
				case FALSE:
				case NOT:
				case IN_TYPE:
				case META_IN_LONG:
				case META_IN_INT:
				case META_IN_SHORT:
				case META_IN_BYTE:
				case META_ADD:
				case META_SUB:
				case META_MUL:
				case META_DIV:
				case META_MOD:
				case META_LESS:
				case META_GREATER:
				case META_LEQ:
				case META_GEQ:
				case META_EQ:
				case META_INT_UNARY_MINUS:
				case META_INT_ADD:
				case META_INT_SUB:
				case META_INT_MUL:
				case META_INT_DIV:
				case META_INT_MOD:
				case META_INT_AND:
				case META_INT_OR:
				case META_INT_XOR:
				case META_INT_SHIFTRIGHT:
				case META_INT_SHIFTLEFT:
				case META_INT_UNSIGNEDSHIFTRIGHT:
				case META_LONG_UNARY_MINUS:
				case META_LONG_ADD:
				case META_LONG_SUB:
				case META_LONG_MUL:
				case META_LONG_DIV:
				case META_LONG_MOD:
				case META_LONG_AND:
				case META_LONG_OR:
				case META_LONG_XOR:
				case META_LONG_SHIFTRIGHT:
				case META_LONG_SHIFTLEFT:
				case META_LONG_UNSIGNEDSHIFTRIGHT:
				case META_MODULO_BYTE:
				case META_MODULO_SHORT:
				case META_MODULO_INTEGER:
				case META_MODULO_LONG:
				case COLON:
				case LPAREN:
				case LBRACE:
				case DIAMOND:
				case BOX:
				case IDENT:
				{
					{
					if ((LA(1)==COLON)) {
						match(COLON);
						s=sort_id();
						if ( inputState.guessing==0 ) {
							
											    bound = true;
											
						}
					}
					else if (((_tokenSet_3.member(LA(1))))&&(func_ns.lookup(new Name(v1.getText()))==null)) {
						if ( inputState.guessing==0 ) {
							
											ts = (TermSymbol) var_ns.lookup(new Name(v1.getText()));
											if (   ts == null
											    || ts instanceof LogicVariable ) {
												throw new SemanticException(
													"Substituted variables need a sort.");
											}
											if ( ! (ts instanceof SchemaVariable &&
												((SchemaVariable)ts).isVariableSV()) ) {
												throw new SemanticException(
												"	"+ts+" is not allowed as bound variable "
													+"in a substitution term.");
											}
											v = (SchemaVariable) ts;
											bound = false;
										
						}
						a1=term();
						match(RBRACE);
						if ( inputState.guessing==0 ) {
							
										  if (bound) {v = bindVar(v1,s);}
						}
						a2=term60();
						if ( inputState.guessing==0 ) {
								
										  a = tf.createSubstitutionTerm(Op.SUBST,v,a1,a2);
										  if (bound) {unbindVars();}
										
						}
					}
					else {
						throw new NoViableAltException(LT(1), getFilename());
					}
					
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				}
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return a;
	}
	
	public final Sort  sort_id() throws RecognitionException, TokenStreamException {
		Sort s = null;
		
		
		s=sort_id_check(true);
		return s;
	}
	
	public final Term  term60() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		Term a1,a2;
			Sort s;
		
		
		switch ( LA(1)) {
		case FORALL:
		case EXISTS:
		case EPSILON:
		case LBRACE:
		{
			a=bindingterm();
			break;
		}
		case DIAMOND:
		case BOX:
		{
			a=dl_term();
			break;
		}
		case NOT:
		{
			match(NOT);
			a1=term60();
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.NOT,new Term[]{a1});
			}
			break;
		}
		default:
			boolean synPredMatched97 = false;
			if (((LA(1)==LPAREN) && (_tokenSet_4.member(LA(2))))) {
				int _m97 = mark();
				synPredMatched97 = true;
				inputState.guessing++;
				try {
					{
					match(LPAREN);
					match(IDENT);
					match(GUARD);
					}
				}
				catch (RecognitionException pe) {
					synPredMatched97 = false;
				}
				rewind(_m97);
				inputState.guessing--;
			}
			if ( synPredMatched97 ) {
				a=condterm();
			}
			else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_5.member(LA(2)))) {
				a=term70();
			}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return a;
	}
	
	public final Term  attrterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
			Vector vec=new Vector();
			ParsableVariable v=null;
			Term t=null;
			ParsableVariable v0=null;
		Sort so=null;
		
		
		
		{
		t=funcpredvarterm();
		if ( inputState.guessing==0 ) {
				so=t.sort();
			
		}
		{
		int _cnt82=0;
		_loop82:
		do {
			if ((LA(1)==DOT)) {
				match(DOT);
				v=attrid(so);
				if ( inputState.guessing==0 ) {
					vec.add(v); so=v.sort();
				}
			}
			else {
				if ( _cnt82>=1 ) { break _loop82; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt82++;
		} while (true);
		}
		}
		if ( inputState.guessing==0 ) {
			
				if (!schematicJava) {
					for (int i=0; i<vec.size(); i++) {
						t=tf.createAttributeTerm((ProgramVariable)vec.get(i),
										t.sort(),t);
					}
				} else {
					t=tf.createAttributeTerm((SchemaVariable)v,t);
				}
				a=t;
			
		}
		return a;
	}
	
	public final Term  funcpredvarterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
			Operator vf;
			int n = 0;
			Vector al = null;
			Term t;
		
		
		vf=varfunc_id();
		{
		if (((_tokenSet_6.member(LA(1))) && (_tokenSet_7.member(LA(2))))&&( vf instanceof ParsableVariable )) {
			if ( inputState.guessing==0 ) {
					a = termForParsedVariable((ParsableVariable)vf);
			}
		}
		else if ((_tokenSet_8.member(LA(1))) && (_tokenSet_7.member(LA(2)))) {
			{
			if (((LA(1)==LPAREN))&&( vf.arity() > 0 )) {
				match(LPAREN);
				t=term();
				if ( inputState.guessing==0 ) {
					
										al = new Vector();
										n=1;
										al.add(t);
									
				}
				{
				_loop119:
				do {
					if ((LA(1)==COMMA)) {
						match(COMMA);
						t=term();
						if ( inputState.guessing==0 ) {
							
													n++;
													al.add(t);
												
						}
					}
					else {
						break _loop119;
					}
					
				} while (true);
				}
				match(RPAREN);
			}
			else if ((_tokenSet_6.member(LA(1)))) {
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			if ( inputState.guessing==0 ) {
				
							    if (vf instanceof Function) {
								if ( al == null ) {	
									a = tf.createFunctionTerm((Function)vf);
								} else {	
									a = tf.createFunctionTerm((Function)vf,
										   (Term[])al.toArray(AN_ARRAY_OF_TERMS));
								}
							    } else if (vf instanceof MetaOperator) {
								
								if ( al == null ) {	
									a = tf.createMetaTerm((MetaOperator)vf, new Term[0]);
								} else {	
									a = tf.createMetaTerm((MetaOperator)vf,
										   (Term[])al.toArray(AN_ARRAY_OF_TERMS));
								}
							    }
							
			}
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
		return a;
	}
	
	public final ParsableVariable  attrid(
		Sort s
	) throws RecognitionException, TokenStreamException {
		ParsableVariable v = null;
		
		Token  attr = null;
		
		
		
		
		attr = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			if (!schematicJava) {
					v = getJavaInfo().getAttribute(attr.getText(), (ObjectSort)s);
				} else {
					v = (SchemaVariable)var_ns.lookup(new Name(attr.getText()));
				}
			
			
		}
		return v;
	}
	
	public final Term  condterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1=null;
		Term a2=null;
		Term a3=null;
		ParsableVariable v=null;
		
		
		match(LPAREN);
		a1=term80();
		match(GUARD);
		a2=term80();
		match(RPAREN);
		match(DOT);
		v=var_id();
		match(CONDUP);
		a3=term60();
		if ( inputState.guessing==0 ) {
			
				AttributeOp aop=null;
				if (((SchemaVariable)v).sort()==ProgramSVSort.VARIABLE) {
				       aop=new AttributeOp((SchemaVariable)v);
				}	
				a = tf.createCondTerm(aop,a1,a2,a3);
			
			
		}
		return a;
	}
	
	public final Term  term80() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		switch ( LA(1)) {
		case LPAREN:
		{
			match(LPAREN);
			a=term();
			match(RPAREN);
			break;
		}
		case TRUE:
		{
			match(TRUE);
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.TRUE);
			}
			break;
		}
		case FALSE:
		{
			match(FALSE);
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.FALSE);
			}
			break;
		}
		default:
			boolean synPredMatched102 = false;
			if (((_tokenSet_1.member(LA(1))) && (LA(2)==COLON||LA(2)==DOT||LA(2)==LPAREN))) {
				int _m102 = mark();
				synPredMatched102 = true;
				inputState.guessing++;
				try {
					{
					match(IDENT);
					match(DOT);
					match(IDENT);
					}
				}
				catch (RecognitionException pe) {
					synPredMatched102 = false;
				}
				rewind(_m102);
				inputState.guessing--;
			}
			if ( synPredMatched102 ) {
				a=attrterm();
			}
			else if ((_tokenSet_1.member(LA(1))) && (_tokenSet_9.member(LA(2)))) {
				a=funcpredvarterm();
			}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return a;
	}
	
	public final Term  term20() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1;
		
		a=term30();
		{
		switch ( LA(1)) {
		case EQV:
		{
			match(EQV);
			a1=term20();
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.EQV,new Term[]{a,a1});
			}
			break;
		}
		case COMMA:
		case RPAREN:
		case RBRACE:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final Term  term30() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1;
		
		a=term40();
		{
		switch ( LA(1)) {
		case IMP:
		{
			match(IMP);
			a1=term30();
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.IMP,new Term[]{a,a1});
			}
			break;
		}
		case COMMA:
		case RPAREN:
		case RBRACE:
		case EQV:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final Term  term40() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1;
		
		a=term50();
		{
		switch ( LA(1)) {
		case OR:
		{
			match(OR);
			a1=term40();
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.OR, new Term[]{a,a1});
			}
			break;
		}
		case COMMA:
		case RPAREN:
		case RBRACE:
		case IMP:
		case EQV:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final Term  term50() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1;
		
		a=term60();
		{
		switch ( LA(1)) {
		case AND:
		{
			match(AND);
			a1=term50();
			if ( inputState.guessing==0 ) {
				a = tf.createJunctorTerm(Op.AND,new Term[]{a,a1});
			}
			break;
		}
		case COMMA:
		case RPAREN:
		case RBRACE:
		case OR:
		case IMP:
		case EQV:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final Term  term70() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Term a1;
		
		a=term80();
		{
		switch ( LA(1)) {
		case EQUALS:
		{
			match(EQUALS);
			a1=term80();
			if ( inputState.guessing==0 ) {
				a = tf.createEqualityTerm(a,a1);
			}
			break;
		}
		case EOF:
		case ASSIGN:
		case COMMA:
		case RPAREN:
		case RBRACE:
		case OR:
		case AND:
		case IMP:
		case EQV:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return a;
	}
	
	public final Term  dl_term() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		
		a=normal_dl_term();
		return a;
	}
	
	public final Term  local_bindingterm() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Token  v0 = null;
		Token  v1 = null;
		
			Operator op = null;
			LogicVariable v=null;
			TermSymbol pv = null;
			Term a0=null;
			Term a1,a2;
			Sort s;
		
		
		switch ( LA(1)) {
		case FORALL:
		case EXISTS:
		case EPSILON:
		{
			{
			switch ( LA(1)) {
			case EPSILON:
			{
				match(EPSILON);
				if ( inputState.guessing==0 ) {
					op = Op.EPSILON;
				}
				break;
			}
			case FORALL:
			{
				match(FORALL);
				if ( inputState.guessing==0 ) {
					op = Op.ALL;
				}
				break;
			}
			case EXISTS:
			{
				match(EXISTS);
				if ( inputState.guessing==0 ) {
					op = Op.EX;
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			v0 = LT(1);
			match(IDENT);
			match(COLON);
			s=sort_id();
			match(DOT);
			if ( inputState.guessing==0 ) {
				v=bindVar(v0,s);
			}
			a1=term60();
			if ( inputState.guessing==0 ) {
				
							if ( op == Op.EPSILON ) {
								a = tf.createEpsilonTerm(v,a1);
							} else {
								a = tf.createQuantifierTerm((Quantifier)op, v,a1);
							}
							unbindVars();
						
			}
			break;
		}
		case LBRACE:
		{
			match(LBRACE);
			{
			boolean synPredMatched107 = false;
			if (((_tokenSet_3.member(LA(1))) && (_tokenSet_10.member(LA(2))))) {
				int _m107 = mark();
				synPredMatched107 = true;
				inputState.guessing++;
				try {
					{
					term60();
					match(ASSIGN);
					}
				}
				catch (RecognitionException pe) {
					synPredMatched107 = false;
				}
				rewind(_m107);
				inputState.guessing--;
			}
			if ( synPredMatched107 ) {
				{
				a0=term60();
				match(ASSIGN);
				a1=term();
				match(RBRACE);
				a2=term60();
				if ( inputState.guessing==0 ) {
					
								a = tf.createUpdateTerm(a0,a1,a2);			
							
				}
				}
			}
			else if ((LA(1)==IDENT) && (LA(2)==COLON||LA(2)==ASSIGN)) {
				{
				v1 = LT(1);
				match(IDENT);
				{
				switch ( LA(1)) {
				case ASSIGN:
				{
					{
					match(ASSIGN);
					a1=term();
					match(RBRACE);
					if ( inputState.guessing==0 ) {
							pv = progVar(v1,a1.sort());
					}
					a2=term60();
					if ( inputState.guessing==0 ) {
						
										if (pv instanceof ProgramVariable) {
											a = tf.createUpdateTerm((ProgramVariable)pv,a1,a2);
										} else {
											a = tf.createUpdateTerm((SchemaVariable)pv,a1,a2);
										} 
									
					}
					}
					break;
				}
				case COLON:
				{
					{
					match(COLON);
					s=sort_id();
					a1=term();
					match(RBRACE);
					if ( inputState.guessing==0 ) {
						v = bindVar(v1,s);
					}
					a2=term60();
					if ( inputState.guessing==0 ) {
						
										a = tf.createSubstitutionTerm(Op.SUBST,v,a1,a2);
										unbindVars();
								      	
					}
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				}
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return a;
	}
	
	public final Term  normal_dl_term() throws RecognitionException, TokenStreamException {
		Term a = null;
		
		Token  d1 = null;
		Token  d2 = null;
			JavaBlock jb=null;
			Term a1;
		
		
		switch ( LA(1)) {
		case DIAMOND:
		{
			d1 = LT(1);
			match(DIAMOND);
			if ( inputState.guessing==0 ) {
				
							jb=getJavaBlock(d1);
							HashSet progVars=progVars(jb);
							bindProgVars(progVars);
						
			}
			a1=term60();
			if ( inputState.guessing==0 ) {
				
							a = tf.createDiamondTerm(jb, a1);
							unbindProgVars();
						
			}
			break;
		}
		case BOX:
		{
			d2 = LT(1);
			match(BOX);
			if ( inputState.guessing==0 ) {
				
							jb=getJavaBlock(d2);
							HashSet progVars=progVars(jb);
							bindProgVars(progVars);
						
			}
			a1=term60();
			if ( inputState.guessing==0 ) {
				
							a = tf.createBoxTerm(jb, a1);
							unbindProgVars();
						
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return a;
	}
	
	public final Operator  varfunc_id() throws RecognitionException, TokenStreamException {
		Operator v = null;
		
		
			String varfunc_name = null;
		
		
		{
		switch ( LA(1)) {
		case IN_TYPE:
		{
			v=expr_op();
			break;
		}
		case IDENT:
		{
			varfunc_name=id();
			if ( inputState.guessing==0 ) {
				
							v = (TermSymbol) var_ns.lookup(new Name(varfunc_name));
							if (v == null) {
								v= (Function) func_ns.lookup(new Name(varfunc_name));
								if (v == null) {
								  throw new NotDeclException
								      ("variable or function", varfunc_name, 
								       getFilename(), getLine(), getColumn());
								}
				
							}
						
			}
			break;
		}
		case META_INT_UNARY_MINUS:
		{
			match(META_INT_UNARY_MINUS);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_UNARY_MINUS ;
			}
			break;
		}
		case META_LONG_UNARY_MINUS:
		{
			match(META_LONG_UNARY_MINUS);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_UNARY_MINUS ;
			}
			break;
		}
		case META_ADD:
		{
			match(META_ADD);
			if ( inputState.guessing==0 ) {
				v = Op.META_ADD;
			}
			break;
		}
		case META_SUB:
		{
			match(META_SUB);
			if ( inputState.guessing==0 ) {
				v = Op.META_SUB;
			}
			break;
		}
		case META_MUL:
		{
			match(META_MUL);
			if ( inputState.guessing==0 ) {
				v = Op.META_MUL;
			}
			break;
		}
		case META_DIV:
		{
			match(META_DIV);
			if ( inputState.guessing==0 ) {
				v = Op.META_DIV;
			}
			break;
		}
		case META_MOD:
		{
			match(META_MOD);
			if ( inputState.guessing==0 ) {
				v = Op.META_MOD;
			}
			break;
		}
		case META_LESS:
		{
			match(META_LESS);
			if ( inputState.guessing==0 ) {
				v = Op.META_LESS;
			}
			break;
		}
		case META_GREATER:
		{
			match(META_GREATER);
			if ( inputState.guessing==0 ) {
				v = Op.META_GREATER;
			}
			break;
		}
		case META_LEQ:
		{
			match(META_LEQ);
			if ( inputState.guessing==0 ) {
				v = Op.META_LEQ;
			}
			break;
		}
		case META_GEQ:
		{
			match(META_GEQ);
			if ( inputState.guessing==0 ) {
				v = Op.META_GEQ;
			}
			break;
		}
		case META_EQ:
		{
			match(META_EQ);
			if ( inputState.guessing==0 ) {
				v = Op.META_EQ;
			}
			break;
		}
		case META_INT_ADD:
		{
			match(META_INT_ADD);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_ADD;
			}
			break;
		}
		case META_INT_SUB:
		{
			match(META_INT_SUB);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_SUB;
			}
			break;
		}
		case META_INT_MUL:
		{
			match(META_INT_MUL);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_MUL;
			}
			break;
		}
		case META_INT_DIV:
		{
			match(META_INT_DIV);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_DIV;
			}
			break;
		}
		case META_INT_MOD:
		{
			match(META_INT_MOD);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_MOD;
			}
			break;
		}
		case META_INT_AND:
		{
			match(META_INT_AND);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_AND;
			}
			break;
		}
		case META_INT_OR:
		{
			match(META_INT_OR);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_OR;
			}
			break;
		}
		case META_INT_XOR:
		{
			match(META_INT_XOR);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_XOR;
			}
			break;
		}
		case META_INT_SHIFTRIGHT:
		{
			match(META_INT_SHIFTRIGHT);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_SHIFTRIGHT;
			}
			break;
		}
		case META_INT_SHIFTLEFT:
		{
			match(META_INT_SHIFTLEFT);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_SHIFTLEFT;
			}
			break;
		}
		case META_INT_UNSIGNEDSHIFTRIGHT:
		{
			match(META_INT_UNSIGNEDSHIFTRIGHT);
			if ( inputState.guessing==0 ) {
				v = Op.META_INT_UNSIGNEDSHIFTRIGHT;
			}
			break;
		}
		case META_LONG_ADD:
		{
			match(META_LONG_ADD);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_ADD;
			}
			break;
		}
		case META_LONG_SUB:
		{
			match(META_LONG_SUB);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_SUB;
			}
			break;
		}
		case META_LONG_MUL:
		{
			match(META_LONG_MUL);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_MUL;
			}
			break;
		}
		case META_LONG_DIV:
		{
			match(META_LONG_DIV);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_DIV;
			}
			break;
		}
		case META_LONG_MOD:
		{
			match(META_LONG_MOD);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_MOD;
			}
			break;
		}
		case META_LONG_AND:
		{
			match(META_LONG_AND);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_AND;
			}
			break;
		}
		case META_LONG_OR:
		{
			match(META_LONG_OR);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_OR;
			}
			break;
		}
		case META_LONG_XOR:
		{
			match(META_LONG_XOR);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_XOR;
			}
			break;
		}
		case META_LONG_SHIFTRIGHT:
		{
			match(META_LONG_SHIFTRIGHT);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_SHIFTRIGHT;
			}
			break;
		}
		case META_LONG_SHIFTLEFT:
		{
			match(META_LONG_SHIFTLEFT);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_SHIFTLEFT;
			}
			break;
		}
		case META_LONG_UNSIGNEDSHIFTRIGHT:
		{
			match(META_LONG_UNSIGNEDSHIFTRIGHT);
			if ( inputState.guessing==0 ) {
				v = Op.META_LONG_UNSIGNEDSHIFTRIGHT;
			}
			break;
		}
		case META_IN_LONG:
		{
			match(META_IN_LONG);
			if ( inputState.guessing==0 ) {
				v = Op.META_IN_LONG;
			}
			break;
		}
		case META_IN_INT:
		{
			match(META_IN_INT);
			if ( inputState.guessing==0 ) {
				v = Op.META_IN_INT;
			}
			break;
		}
		case META_IN_SHORT:
		{
			match(META_IN_SHORT);
			if ( inputState.guessing==0 ) {
				v = Op.META_IN_SHORT;
			}
			break;
		}
		case META_IN_BYTE:
		{
			match(META_IN_BYTE);
			if ( inputState.guessing==0 ) {
				v = Op.META_IN_BYTE;
			}
			break;
		}
		case META_MODULO_BYTE:
		{
			match(META_MODULO_BYTE);
			if ( inputState.guessing==0 ) {
				v = Op.META_MODULO_BYTE;
			}
			break;
		}
		case META_MODULO_SHORT:
		{
			match(META_MODULO_SHORT);
			if ( inputState.guessing==0 ) {
				v = Op.META_MODULO_SHORT;
			}
			break;
		}
		case META_MODULO_INTEGER:
		{
			match(META_MODULO_INTEGER);
			if ( inputState.guessing==0 ) {
				v = Op.META_MODULO_INTEGER;
			}
			break;
		}
		case META_MODULO_LONG:
		{
			match(META_MODULO_LONG);
			if ( inputState.guessing==0 ) {
				v = Op.META_MODULO_LONG;
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return v;
	}
	
	public final Operator  expr_op() throws RecognitionException, TokenStreamException {
		Operator v = null;
		
		Token  id = null;
		Token  id1 = null;
		Token  id2 = null;
		
			ParsableVariable left_var0 = null;
			ParsableVariable right_var0 = null;
			ParsableVariable unary_var0 = null;
		
		
		match(IN_TYPE);
		match(LPAREN);
		{
		switch ( LA(1)) {
		case IDENT:
		{
			{
			left_var0=var_id();
			{
			switch ( LA(1)) {
			case SLASH:
			case ARITH_OP:
			case IDENT:
			{
				{
				switch ( LA(1)) {
				case ARITH_OP:
				{
					id = LT(1);
					match(ARITH_OP);
					break;
				}
				case SLASH:
				{
					id1 = LT(1);
					match(SLASH);
					break;
				}
				case IDENT:
				{
					id2 = LT(1);
					match(IDENT);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				right_var0=var_id();
				break;
			}
			case RPAREN:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			}
			break;
		}
		case 148:
		{
			{
			match(148);
			unary_var0=var_id();
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(RPAREN);
		if ( inputState.guessing==0 ) {
			
			if (id==null) id=id1;	
				   if (unary_var0!=null) {
				      // can only be unary minus
				      SchemaVariable unary_var = (SchemaVariable) unary_var0;
				      Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
				      return new InTypeOperator
					(type, new de.uka.ilkd.key.java.expression.operator.Negative(unary_var));
				   }
				   else {
				      SchemaVariable left_var = (SchemaVariable) left_var0;
				      Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
				      if (right_var0 != null) {
				         SchemaVariable right_var = (SchemaVariable) right_var0;
				         String op_str = (id == null ? id2.getText() : id.getText());
				   if ("+".equals(op_str)) {
					     return new InTypeOperator
					       (type, new de.uka.ilkd.key.java.expression.operator.Plus(left_var, right_var));
				         } else if ("-".equals(op_str)) {
					     return new InTypeOperator
					       (type, new de.uka.ilkd.key.java.expression.operator.Minus(left_var, right_var));
				         } else if ("*".equals(op_str)) {
					     return new InTypeOperator
					       (type, new de.uka.ilkd.key.java.expression.operator.Times(left_var, right_var));
				         } else if ("/".equals(op_str)) {
				    	     return new InTypeOperator
					       (type, new de.uka.ilkd.key.java.expression.operator.Divide(left_var, right_var));
					} else if ("%".equals(op_str)) {
				    	     return new InTypeOperator
					       (type, new de.uka.ilkd.key.java.expression.operator.Modulo(left_var, right_var));
				         } 
				      }
				      else {
				         return new InTypeOperator(type, left_var);
				      }
				   }
				
		}
		return v;
	}
	
	public final String  id() throws RecognitionException, TokenStreamException {
		String name = null;
		
		Token  classID = null;
		Token  nameID = null;
		
		classID = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
						name = classID.getText();
					
		}
		{
		switch ( LA(1)) {
		case COLON:
		{
			match(COLON);
			match(COLON);
			nameID = LT(1);
			match(IDENT);
			if ( inputState.guessing==0 ) {
				
							 name = classID.getText() + 
							        (nameID == null ? "" : 
							        "::" + nameID.getText());
							
			}
			break;
		}
		case EOF:
		case SEMI:
		case GUARD:
		case ASSIGN:
		case DOT:
		case COMMA:
		case LPAREN:
		case RPAREN:
		case RBRACE:
		case OR:
		case AND:
		case IMP:
		case EQV:
		case EQUALS:
		case SEQARROW:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		return name;
	}
	
	public final void sort_decls() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
			boolean isObjectSort=false;
		
		
		match(SORTS);
		match(LBRACE);
		{
		_loop135:
		do {
			if ((LA(1)==OBJECT||LA(1)==IDENT)) {
				{
				switch ( LA(1)) {
				case OBJECT:
				{
					match(OBJECT);
					if ( inputState.guessing==0 ) {
						isObjectSort=true;
					}
					break;
				}
				case IDENT:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				id = LT(1);
				match(IDENT);
				match(SEMI);
				if ( inputState.guessing==0 ) {
					
								    Name sort_name = new Name(id.getText());
								    if (sort_ns.lookup(sort_name) == null) {
									Sort s;
									if (!isObjectSort) {
										s = new PrimitiveSort(sort_name);
									} else {
								 	     if ("Object".equals(sort_name)) {
									         if (sort_ns.lookup(new Name("Object"))
													      ==null) {
					
											 s = new ObjectSort(sort_name);
										  }
										  s=(Sort)sort_ns.lookup(new Name("Object"));
										} else {
											s = new ObjectSort(sort_name, 
										                 (Sort)sort_ns.lookup
					(new Name("Object")));
										}	
									}
									sort_ns.add(s);
									if (s instanceof NonCollectionSort) {
										NonCollectionSort ns
											= (NonCollectionSort)s;
										sort_ns.add(ns.getSetSort());
										sort_ns.add(ns.getSequenceSort());
										sort_ns.add(ns.getBagSort());
									}
								    }
								
				}
			}
			else {
				break _loop135;
			}
			
		} while (true);
		}
		match(RBRACE);
	}
	
	public final void schema_var_decls() throws RecognitionException, TokenStreamException {
		
		
			Sort s = null;
			boolean makeVariableSV = false;
		
		
		match(SCHEMA);
		match(VARIABLES);
		match(LBRACE);
		{
		_loop149:
		do {
			if ((LA(1)==PROGRAM||LA(1)==FORMULA||LA(1)==IDENT)) {
				if ( inputState.guessing==0 ) {
					makeVariableSV = false;
				}
				{
				switch ( LA(1)) {
				case PROGRAM:
				{
					{
					match(PROGRAM);
					{
					switch ( LA(1)) {
					case STATEMENT:
					{
						match(STATEMENT);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.STATEMENT;
												
						}
						break;
					}
					case STATEMENTLIST:
					{
						match(STATEMENTLIST);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.STATEMENTLIST;
												
						}
						break;
					}
					case EXPRESSION:
					{
						match(EXPRESSION);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.EXPRESSION;
												
						}
						break;
					}
					case EXPRESSIONLIST:
					{
						match(EXPRESSIONLIST);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.EXPRESSIONLIST;
												
						}
						break;
					}
					case JAVABOOL:
					{
						match(JAVABOOL);
						match(EXPRESSION);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.BOOLEANEXPRESSION;
												
						}
						break;
					}
					case LEFTHANDSIDE:
					{
						match(LEFTHANDSIDE);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.LEFTHANDSIDE;
												
						}
						break;
					}
					case FORINIT:
					{
						match(FORINIT);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.FORINIT;
												
						}
						break;
					}
					case STATEMENTEXPRESSIONLIST:
					{
						match(STATEMENTEXPRESSIONLIST);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.STATEMENTEXPRESSIONLIST;
												
						}
						break;
					}
					case LABEL:
					{
						match(LABEL);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.LABEL;
												
						}
						break;
					}
					case TYPE:
					{
						match(TYPE);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.TYPE;
												
						}
						break;
					}
					case VARIABLE:
					{
						match(VARIABLE);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.VARIABLE;
												
						}
						break;
					}
					case VARIABLEINIT:
					{
						match(VARIABLEINIT);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.VARIABLEINIT;
												
						}
						break;
					}
					case NEW:
					{
						match(NEW);
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.NEW;
												
						}
						break;
					}
					case BOOLEANVARIABLE:
					{
						{
						match(BOOLEANVARIABLE);
						}
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.BOOLEANVARIABLE;
												
						}
						break;
					}
					case CATCHFINALLY:
					{
						{
						match(CATCHFINALLY);
						}
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.CATCHFINALLY;
												
						}
						break;
					}
					case METHODREFERENCE:
					{
						{
						match(METHODREFERENCE);
						}
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.METHODREFERENCE;
												
						}
						break;
					}
					case METHODBODY:
					{
						{
						match(METHODBODY);
						}
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.METHODBODY;
												
						}
						break;
					}
					case BRANCHLIST:
					{
						{
						match(BRANCHLIST);
						}
						if ( inputState.guessing==0 ) {
							
													s=ProgramSVSort.BRANCHLIST;
												
						}
						break;
					}
					default:
						if ((LA(1)==SIMPLE) && (LA(2)==EXPRESSION)) {
							match(SIMPLE);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVABOOL)) {
							match(SIMPLE);
							match(JAVABOOL);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVABOOLEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVABYTE)) {
							match(SIMPLE);
							match(JAVABYTE);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVABYTEEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVASHORT)) {
							match(SIMPLE);
							match(JAVASHORT);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVASHORTEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVAINT)) {
							match(SIMPLE);
							match(JAVAINT);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVAINTEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVALONG)) {
							match(SIMPLE);
							match(JAVALONG);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVALONGEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==JAVABYTESHORTINT)) {
							match(SIMPLE);
							match(JAVABYTESHORTINT);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEJAVABYTESHORTINTEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==ANYJAVATYPE)) {
							match(SIMPLE);
							match(ANYJAVATYPE);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLEANYJAVATYPEEXPRESSION;
													
							}
						}
						else if ((LA(1)==NONSIMPLE) && (LA(2)==EXPRESSION)) {
							match(NONSIMPLE);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.NONSIMPLEEXPRESSION;
													
							}
						}
						else if ((LA(1)==NONSIMPLE) && (LA(2)==JAVABOOL)) {
							match(NONSIMPLE);
							match(JAVABOOL);
							match(EXPRESSION);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.NONSIMPLEBOOLEANEXPRESSION;
													
							}
						}
						else if ((LA(1)==SIMPLE) && (LA(2)==LEFTHANDSIDE)) {
							match(SIMPLE);
							match(LEFTHANDSIDE);
							if ( inputState.guessing==0 ) {
								
														s=ProgramSVSort.SIMPLELEFTHANDSIDE;
													
							}
						}
					else {
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					}
					break;
				}
				case FORMULA:
				{
					match(FORMULA);
					if ( inputState.guessing==0 ) {
						
											s = Sort.FORMULA;
										
					}
					break;
				}
				case IDENT:
				{
					s=sort_id_check(true);
					{
					switch ( LA(1)) {
					case VARIABLES:
					{
						match(VARIABLES);
						if ( inputState.guessing==0 ) {
							makeVariableSV = true;
						}
						break;
					}
					case IDENT:
					{
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				one_schema_var_decl(s,makeVariableSV);
				{
				_loop148:
				do {
					if ((LA(1)==COMMA)) {
						match(COMMA);
						one_schema_var_decl(s,makeVariableSV);
					}
					else {
						break _loop148;
					}
					
				} while (true);
				}
				match(SEMI);
			}
			else {
				break _loop149;
			}
			
		} while (true);
		}
		match(RBRACE);
	}
	
	public final void pred_decls() throws RecognitionException, TokenStreamException {
		
		
			Sort[] argSorts;
			String pred_name;
		
		
		match(PREDICATES);
		match(LBRACE);
		{
		_loop153:
		do {
			if ((LA(1)==IDENT)) {
				pred_name=id();
				argSorts=arg_sorts(!skip_predicates);
				if ( inputState.guessing==0 ) {
					
								    if (!skip_predicates) {
									Name predicate = new Name(pred_name);
									if (func_ns.lookup(predicate) != null) {
									    // throw new AmbigiousDeclException
									    //("predicate "+pred_name+" ", 
									    //getLine(), getColumn());
									} else {
									    Function p = new Function(predicate,
												      Sort.FORMULA, 
												      argSorts);
									    func_ns.add(p);			
									}
								    } 
								
				}
				match(SEMI);
			}
			else {
				break _loop153;
			}
			
		} while (true);
		}
		match(RBRACE);
	}
	
	public final void func_decls() throws RecognitionException, TokenStreamException {
		
		
			Sort[] argSorts;
			Sort retSort;
		String func_name;
		
		
		match(FUNCTIONS);
		match(LBRACE);
		{
		_loop156:
		do {
			if ((LA(1)==IDENT)) {
				retSort=sort_id_check(!skip_functions);
				func_name=id();
				argSorts=arg_sorts(!skip_functions);
				if ( inputState.guessing==0 ) {
					
								    if (!skip_functions) {
					
									Name fct_name = new Name(func_name);
									if (func_ns.lookup(fct_name) != null) {
									    //throw new AmbigiousDeclException
									    //("function "+func_name, 
									    //getLine(), getColumn());
									} else {
									    Function f=new Function(fct_name,
												    retSort, 
												    argSorts);
									    func_ns.add(f);
									}
								    } 
								
				}
				match(SEMI);
			}
			else {
				break _loop156;
			}
			
		} while (true);
		}
		match(RBRACE);
	}
	
	public final void heur_decls() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		match(HEURISTICS);
		match(LBRACE);
		{
		_loop163:
		do {
			if ((LA(1)==IDENT)) {
				id = LT(1);
				match(IDENT);
				match(SEMI);
				if ( inputState.guessing==0 ) {
					
								    if (!skip_heuristics) {
									Heuristic h = new Heuristic(new Name(id.getText()));
									heur_ns.add(h);
								    }
								
				}
			}
			else {
				break _loop163;
			}
			
		} while (true);
		}
		match(RBRACE);
	}
	
	public final void one_schema_var_decl(
		Sort s, boolean makeVariableSV
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		id = LT(1);
		match(IDENT);
		if ( inputState.guessing==0 ) {
			
					    if (!skip_schemavariables) {
						SchemaVariable v;
						if ( s == Sort.FORMULA ) {
							v = SchemaVariable.createFormulaSV(
									       new Name(id.getText())
									);
						} else if ( s instanceof ProgramSVSort ) {
							v = SchemaVariable.createProgramSV(
									new Name(id.getText())
									,(ProgramSVSort) s
									);
						} else {
							if ( makeVariableSV ) {
								v=SchemaVariable.createVariableSV(
								               new Name(id.getText()), s);
							} else {
								v = SchemaVariable.createTermSV(
									new Name(id.getText()), s);
							}
						}
						var_ns.add(v);
					    }
					
		}
	}
	
	public final Sort[]  arg_sorts(
		boolean checkSort
	) throws RecognitionException, TokenStreamException {
		Sort[] argSorts = null;
		
		
			Vector args = new Vector();
			Sort s;
		
		
		{
		switch ( LA(1)) {
		case LPAREN:
		{
			match(LPAREN);
			s=sort_id_check(checkSort);
			if ( inputState.guessing==0 ) {
				args.add(s);
			}
			{
			_loop160:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					s=sort_id_check(checkSort);
					if ( inputState.guessing==0 ) {
						args.add(s);
					}
				}
				else {
					break _loop160;
				}
				
			} while (true);
			}
			match(RPAREN);
			break;
		}
		case SEMI:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			
						argSorts = (Sort[])args.toArray(AN_ARRAY_OF_SORTS);
					
		}
		return argSorts;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"java\"",
		"\"statement\"",
		"\"statementlist\"",
		"\"expression\"",
		"\"expressionlist\"",
		"\"nonsimple\"",
		"\"lefthandside\"",
		"\"simple\"",
		"\"notsimple\"",
		"\"forinit\"",
		"\"typeof\"",
		"\"type\"",
		"\"javaboolean\"",
		"\"object\"",
		"\"javabyte\"",
		"\"javashort\"",
		"\"javaint\"",
		"\"javalong\"",
		"\"javabyteshortint\"",
		"\"anyjavatype\"",
		"\"number\"",
		"\"intliteral\"",
		"\"longliteral\"",
		"\"variable\"",
		"\"booleanvariable\"",
		"\"catchfinally\"",
		"\"methodreference\"",
		"\"methodbody\"",
		"\"variableinitializer\"",
		"\"statementexpressionlist\"",
		"\"label\"",
		"\"program\"",
		"\"all\"",
		"\"ex\"",
		"\"eps\"",
		"\"true\"",
		"\"false\"",
		"\"find\"",
		"\"if\"",
		"\"varcond\"",
		"\"noninteractive\"",
		"\"recursive\"",
		"\"displayname\"",
		"\"helptext\"",
		"\"new\"",
		"\"not\"",
		"\"free\"",
		"\"in\"",
		"\"close\"",
		"\"goal\"",
		"\"replacewith\"",
		"\"add\"",
		"\"addrules\"",
		"\"addprogvars\"",
		"\"heuristics\"",
		"\"sorts\"",
		"\"schema\"",
		"\"variables\"",
		"\"predicates\"",
		"\"functions\"",
		"\"formula\"",
		"\"rules\"",
		"\"branchlist\"",
		"\"proof\"",
		"\"rule\"",
		"\"term\"",
		"\"branch\"",
		"\"problem\"",
		"\"#inType\"",
		"\"#inLong\"",
		"\"#inInt\"",
		"\"#inShort\"",
		"\"#inByte\"",
		"\"#add\"",
		"\"#sub\"",
		"\"#mul\"",
		"\"#div\"",
		"\"#mod\"",
		"\"#less\"",
		"\"#greater\"",
		"\"#leq\"",
		"\"#geq\"",
		"\"#eq\"",
		"\"#JavaIntUnaryMinus\"",
		"\"#JavaIntAdd\"",
		"\"#JavaIntSub\"",
		"\"#JavaIntMul\"",
		"\"#JavaIntDiv\"",
		"\"#JavaIntMod\"",
		"\"#JavaIntAnd\"",
		"\"#JavaIntOr\"",
		"\"#JavaIntXor\"",
		"\"#JavaIntShiftRight\"",
		"\"#JavaIntShiftLeft\"",
		"\"#JavaIntUnsignedShiftRight\"",
		"\"#JavaLongUnaryMinus\"",
		"\"#JavaLongAdd\"",
		"\"#JavaLongSub\"",
		"\"#JavaLongMul\"",
		"\"#JavaLongDiv\"",
		"\"#JavaLongMod\"",
		"\"#JavaLongAnd\"",
		"\"#JavaLongOr\"",
		"\"#JavaLongXor\"",
		"\"#JavaLongShiftRight\"",
		"\"#JavaLongShiftLeft\"",
		"\"#JavaLongUnsignedShiftRight\"",
		"\"#moduloByte\"",
		"\"#moduloShort\"",
		"\"#moduloInteger\"",
		"\"#moduloLong\"",
		"VOCAB",
		"`;'",
		"`/'",
		"`:'",
		"`?='",
		"`/->'",
		"`:='",
		"`.'",
		"`,'",
		"`('",
		"`)'",
		"`{'",
		"`}'",
		"`|'",
		"`&'",
		"`->'",
		"`<->'",
		"`='",
		"`==>'",
		"java operator",
		"white space",
		"a string in double quotes",
		"ESC",
		"DIAMOND",
		"BOX",
		"BLOCK",
		"a string with double quotes",
		"a comment",
		"a comment",
		"a digit",
		"a letter",
		"an admissible character for identifiers",
		"an identifer",
		"\"-\""
	};
	
	private static final long _tokenSet_0_data_[] = { 0L, 113L, 524288L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_0 = new BitSet(_tokenSet_0_data_);
	private static final long _tokenSet_1_data_[] = { 0L, 2251799813684992L, 524288L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_1 = new BitSet(_tokenSet_1_data_);
	private static final long _tokenSet_2_data_[] = { 565080257200128L, 5928988909433257728L, 527360L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_2 = new BitSet(_tokenSet_2_data_);
	private static final long _tokenSet_3_data_[] = { 565080257200128L, 5766859322847919872L, 527360L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_3 = new BitSet(_tokenSet_3_data_);
	private static final long _tokenSet_4_data_[] = { 1649267441664L, 1155173304420531968L, 524288L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_4 = new BitSet(_tokenSet_4_data_);
	private static final long _tokenSet_5_data_[] = { 565080257200130L, -123848989752688896L, 527423L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_5 = new BitSet(_tokenSet_5_data_);
	private static final long _tokenSet_6_data_[] = { 2L, -5872693914091126784L, 63L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_6 = new BitSet(_tokenSet_6_data_);
	private static final long _tokenSet_7_data_[] = { 563789911585456130L, -101330991615836416L, 527423L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_7 = new BitSet(_tokenSet_7_data_);
	private static final long _tokenSet_8_data_[] = { 2L, -4719772409484279808L, 63L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_8 = new BitSet(_tokenSet_8_data_);
	private static final long _tokenSet_9_data_[] = { 2L, -4989988387126509568L, 63L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_9 = new BitSet(_tokenSet_9_data_);
	private static final long _tokenSet_10_data_[] = { 565080257200128L, 6217219285584969472L, 527376L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_10 = new BitSet(_tokenSet_10_data_);
	
	}
